/*******************************************************************************
* File Name: PD_USBFS_drv.c
* Version 2.80
*
* Description:
*  Endpoint 0 Driver for the USBFS Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PD_USBFS.h"
#include "PD_USBFS_pvt.h"


/***************************************
* Global data allocation
***************************************/

volatile T_PD_USBFS_EP_CTL_BLOCK PD_USBFS_EP[PD_USBFS_MAX_EP];
volatile uint8 PD_USBFS_configuration;
volatile uint8 PD_USBFS_interfaceNumber;
volatile uint8 PD_USBFS_configurationChanged;
volatile uint8 PD_USBFS_deviceAddress;
volatile uint8 PD_USBFS_deviceStatus;
volatile uint8 PD_USBFS_interfaceSetting[PD_USBFS_MAX_INTERFACES_NUMBER];
volatile uint8 PD_USBFS_interfaceSetting_last[PD_USBFS_MAX_INTERFACES_NUMBER];
volatile uint8 PD_USBFS_interfaceStatus[PD_USBFS_MAX_INTERFACES_NUMBER];
volatile uint8 PD_USBFS_device;
const uint8 CYCODE *PD_USBFS_interfaceClass;


/***************************************
* Local data allocation
***************************************/

volatile uint8 PD_USBFS_ep0Toggle;
volatile uint8 PD_USBFS_lastPacketSize;
volatile uint8 PD_USBFS_transferState;
volatile T_PD_USBFS_TD PD_USBFS_currentTD;
volatile uint8 PD_USBFS_ep0Mode;
volatile uint8 PD_USBFS_ep0Count;
volatile uint16 PD_USBFS_transferByteCount;


/*******************************************************************************
* Function Name: PD_USBFS_ep_0_Interrupt
********************************************************************************
*
* Summary:
*  This Interrupt Service Routine handles Endpoint 0 (Control Pipe) traffic.
*  It dispatches setup requests and handles the data and status stages.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
CY_ISR(PD_USBFS_EP_0_ISR)
{
    uint8 bRegTemp;
    uint8 modifyReg;


    bRegTemp = CY_GET_REG8(PD_USBFS_EP0_CR_PTR);
    if ((bRegTemp & PD_USBFS_MODE_ACKD) != 0u)
    {
        modifyReg = 1u;
        if ((bRegTemp & PD_USBFS_MODE_SETUP_RCVD) != 0u)
        {
            if((bRegTemp & PD_USBFS_MODE_MASK) != PD_USBFS_MODE_NAK_IN_OUT)
            {
                modifyReg = 0u;                                     /* When mode not NAK_IN_OUT => invalid setup */
            }
            else
            {
                PD_USBFS_HandleSetup();
                if((PD_USBFS_ep0Mode & PD_USBFS_MODE_SETUP_RCVD) != 0u)
                {
                    modifyReg = 0u;                         /* if SETUP bit set -> exit without modifying the mode */
                }

            }
        }
        else if ((bRegTemp & PD_USBFS_MODE_IN_RCVD) != 0u)
        {
            PD_USBFS_HandleIN();
        }
        else if ((bRegTemp & PD_USBFS_MODE_OUT_RCVD) != 0u)
        {
            PD_USBFS_HandleOUT();
        }
        else
        {
            modifyReg = 0u;
        }
        if(modifyReg != 0u)
        {
            bRegTemp = CY_GET_REG8(PD_USBFS_EP0_CR_PTR);    /* unlock registers */
            if((bRegTemp & PD_USBFS_MODE_SETUP_RCVD) == 0u)  /* Check if SETUP bit is not set, otherwise exit */
            {
                /* Update the count register */
                bRegTemp = PD_USBFS_ep0Toggle | PD_USBFS_ep0Count;
                CY_SET_REG8(PD_USBFS_EP0_CNT_PTR, bRegTemp);
                if(bRegTemp == CY_GET_REG8(PD_USBFS_EP0_CNT_PTR))   /* continue if writing was successful */
                {
                    do
                    {
                        modifyReg = PD_USBFS_ep0Mode;       /* Init temporary variable */
                        /* Unlock registers */
                        bRegTemp = CY_GET_REG8(PD_USBFS_EP0_CR_PTR) & PD_USBFS_MODE_SETUP_RCVD;
                        if(bRegTemp == 0u)                          /* Check if SETUP bit is not set */
                        {
                            /* Set the Mode Register  */
                            CY_SET_REG8(PD_USBFS_EP0_CR_PTR, PD_USBFS_ep0Mode);
                            /* Writing check */
                            modifyReg = CY_GET_REG8(PD_USBFS_EP0_CR_PTR) & PD_USBFS_MODE_MASK;
                        }
                    }while(modifyReg != PD_USBFS_ep0Mode);  /* Repeat if writing was not successful */
                }
            }
        }
    }
}


/*******************************************************************************
* Function Name: PD_USBFS_HandleSetup
********************************************************************************
*
* Summary:
*  This Routine dispatches requests for the four USB request types
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_HandleSetup(void) 
{
    uint8 requestHandled;

    requestHandled = CY_GET_REG8(PD_USBFS_EP0_CR_PTR);      /* unlock registers */
    CY_SET_REG8(PD_USBFS_EP0_CR_PTR, requestHandled);       /* clear setup bit */
    requestHandled = CY_GET_REG8(PD_USBFS_EP0_CR_PTR);      /* reread register */
    if((requestHandled & PD_USBFS_MODE_SETUP_RCVD) != 0u)
    {
        PD_USBFS_ep0Mode = requestHandled;        /* if SETUP bit set -> exit without modifying the mode */
    }
    else
    {
        /* In case the previous transfer did not complete, close it out */
        PD_USBFS_UpdateStatusBlock(PD_USBFS_XFER_PREMATURE);

        switch (CY_GET_REG8(PD_USBFS_bmRequestType) & PD_USBFS_RQST_TYPE_MASK)
        {
            case PD_USBFS_RQST_TYPE_STD:
                requestHandled = PD_USBFS_HandleStandardRqst();
                break;
            case PD_USBFS_RQST_TYPE_CLS:
                requestHandled = PD_USBFS_DispatchClassRqst();
                break;
            case PD_USBFS_RQST_TYPE_VND:
                requestHandled = PD_USBFS_HandleVendorRqst();
                break;
            default:
                requestHandled = PD_USBFS_FALSE;
                break;
        }
        if (requestHandled == PD_USBFS_FALSE)
        {
            PD_USBFS_ep0Mode = PD_USBFS_MODE_STALL_IN_OUT;
        }
    }
}


/*******************************************************************************
* Function Name: PD_USBFS_HandleIN
********************************************************************************
*
* Summary:
*  This routine handles EP0 IN transfers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_HandleIN(void) 
{
    switch (PD_USBFS_transferState)
    {
        case PD_USBFS_TRANS_STATE_IDLE:
            break;
        case PD_USBFS_TRANS_STATE_CONTROL_READ:
            PD_USBFS_ControlReadDataStage();
            break;
        case PD_USBFS_TRANS_STATE_CONTROL_WRITE:
            PD_USBFS_ControlWriteStatusStage();
            break;
        case PD_USBFS_TRANS_STATE_NO_DATA_CONTROL:
            PD_USBFS_NoDataControlStatusStage();
            break;
        default:    /* there are no more states */
            break;
    }
}


/*******************************************************************************
* Function Name: PD_USBFS_HandleOUT
********************************************************************************
*
* Summary:
*  This routine handles EP0 OUT transfers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_HandleOUT(void) 
{
    switch (PD_USBFS_transferState)
    {
        case PD_USBFS_TRANS_STATE_IDLE:
            break;
        case PD_USBFS_TRANS_STATE_CONTROL_READ:
            PD_USBFS_ControlReadStatusStage();
            break;
        case PD_USBFS_TRANS_STATE_CONTROL_WRITE:
            PD_USBFS_ControlWriteDataStage();
            break;
        case PD_USBFS_TRANS_STATE_NO_DATA_CONTROL:
            /* Update the completion block */
            PD_USBFS_UpdateStatusBlock(PD_USBFS_XFER_ERROR);
            /* We expect no more data, so stall INs and OUTs */
            PD_USBFS_ep0Mode = PD_USBFS_MODE_STALL_IN_OUT;
            break;
        default:    /* There are no more states */
            break;
    }
}


/*******************************************************************************
* Function Name: PD_USBFS_LoadEP0
********************************************************************************
*
* Summary:
*  This routine loads the EP0 data registers for OUT transfers.  It uses the
*  currentTD (previously initialized by the _InitControlWrite function and
*  updated for each OUT transfer, and the bLastPacketSize) to determine how
*  many uint8s to transfer on the current OUT.
*
*  If the number of uint8s remaining is zero and the last transfer was full,
*  we need to send a zero length packet.  Otherwise we send the minimum
*  of the control endpoint size (8) or remaining number of uint8s for the
*  transaction.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_transferByteCount - Update the transfer byte count from the
*     last transaction.
*  PD_USBFS_ep0Count - counts the data loaded to the SIE memory in
*     current packet.
*  PD_USBFS_lastPacketSize - remembers the USBFS_ep0Count value for the
*     next packet.
*  PD_USBFS_transferByteCount - sum of the previous bytes transferred
*     on previous packets(sum of USBFS_lastPacketSize)
*  PD_USBFS_ep0Toggle - inverted
*  PD_USBFS_ep0Mode  - prepare for mode register content.
*  PD_USBFS_transferState - set to TRANS_STATE_CONTROL_READ
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_LoadEP0(void) 
{
    uint8 ep0Count = 0u;

    /* Update the transfer byte count from the last transaction */
    PD_USBFS_transferByteCount += PD_USBFS_lastPacketSize;
    /* Now load the next transaction */
    while ((PD_USBFS_currentTD.count > 0u) && (ep0Count < 8u))
    {
        CY_SET_REG8((reg8 *)(PD_USBFS_EP0_DR0_IND + ep0Count), *PD_USBFS_currentTD.pData);
        PD_USBFS_currentTD.pData = &PD_USBFS_currentTD.pData[1u];
        ep0Count++;
        PD_USBFS_currentTD.count--;
    }
    /* Support zero-length packet*/
    if( (PD_USBFS_lastPacketSize == 8u) || (ep0Count > 0u) )
    {
        /* Update the data toggle */
        PD_USBFS_ep0Toggle ^= PD_USBFS_EP0_CNT_DATA_TOGGLE;
        /* Set the Mode Register  */
        PD_USBFS_ep0Mode = PD_USBFS_MODE_ACK_IN_STATUS_OUT;
        /* Update the state (or stay the same) */
        PD_USBFS_transferState = PD_USBFS_TRANS_STATE_CONTROL_READ;
    }
    else
    {
        /* Expect Status Stage Out */
        PD_USBFS_ep0Mode = PD_USBFS_MODE_STATUS_OUT_ONLY;
        /* Update the state (or stay the same) */
        PD_USBFS_transferState = PD_USBFS_TRANS_STATE_CONTROL_READ;
    }

    /* Save the packet size for next time */
    PD_USBFS_lastPacketSize = ep0Count;
    PD_USBFS_ep0Count = ep0Count;
}


/*******************************************************************************
* Function Name: PD_USBFS_InitControlRead
********************************************************************************
*
* Summary:
*  Initialize a control read transaction, usable to send data to the host.
*  The following global variables should be initialized before this function
*  called. To send zero length packet use InitZeroLengthControlTransfer
*  function.
*
* Parameters:
*  None.
*
* Return:
*  requestHandled state.
*
* Global variables:
*  PD_USBFS_currentTD.count - counts of data to be sent.
*  PD_USBFS_currentTD.pData - data pointer.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 PD_USBFS_InitControlRead(void) 
{
    uint16 xferCount;
    if(PD_USBFS_currentTD.count == 0u)
    {
        (void) PD_USBFS_InitZeroLengthControlTransfer();
    }
    else
    {
        /* Set up the state machine */
        PD_USBFS_transferState = PD_USBFS_TRANS_STATE_CONTROL_READ;
        /* Set the toggle, it gets updated in LoadEP */
        PD_USBFS_ep0Toggle = 0u;
        /* Initialize the Status Block */
        PD_USBFS_InitializeStatusBlock();
        xferCount = (((uint16)CY_GET_REG8(PD_USBFS_lengthHi) << 8u) | (CY_GET_REG8(PD_USBFS_lengthLo)));

        if (PD_USBFS_currentTD.count > xferCount)
        {
            PD_USBFS_currentTD.count = xferCount;
        }
        PD_USBFS_LoadEP0();
    }

    return(PD_USBFS_TRUE);
}


/*******************************************************************************
* Function Name: PD_USBFS_InitZeroLengthControlTransfer
********************************************************************************
*
* Summary:
*  Initialize a zero length data IN transfer.
*
* Parameters:
*  None.
*
* Return:
*  requestHandled state.
*
* Global variables:
*  PD_USBFS_ep0Toggle - set to EP0_CNT_DATA_TOGGLE
*  PD_USBFS_ep0Mode  - prepare for mode register content.
*  PD_USBFS_transferState - set to TRANS_STATE_CONTROL_READ
*  PD_USBFS_ep0Count - cleared, means the zero-length packet.
*  PD_USBFS_lastPacketSize - cleared.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 PD_USBFS_InitZeroLengthControlTransfer(void)
                                                
{
    /* Update the state */
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_CONTROL_READ;
    /* Set the data toggle */
    PD_USBFS_ep0Toggle = PD_USBFS_EP0_CNT_DATA_TOGGLE;
    /* Set the Mode Register  */
    PD_USBFS_ep0Mode = PD_USBFS_MODE_ACK_IN_STATUS_OUT;
    /* Save the packet size for next time */
    PD_USBFS_lastPacketSize = 0u;
    PD_USBFS_ep0Count = 0u;

    return(PD_USBFS_TRUE);
}


/*******************************************************************************
* Function Name: PD_USBFS_ControlReadDataStage
********************************************************************************
*
* Summary:
*  Handle the Data Stage of a control read transfer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_ControlReadDataStage(void) 

{
    PD_USBFS_LoadEP0();
}


/*******************************************************************************
* Function Name: PD_USBFS_ControlReadStatusStage
********************************************************************************
*
* Summary:
*  Handle the Status Stage of a control read transfer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_USBFS_transferByteCount - updated with last packet size.
*  PD_USBFS_transferState - set to TRANS_STATE_IDLE.
*  PD_USBFS_ep0Mode  - set to MODE_STALL_IN_OUT.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_ControlReadStatusStage(void) 
{
    /* Update the transfer byte count */
    PD_USBFS_transferByteCount += PD_USBFS_lastPacketSize;
    /* Go Idle */
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_IDLE;
    /* Update the completion block */
    PD_USBFS_UpdateStatusBlock(PD_USBFS_XFER_STATUS_ACK);
    /* We expect no more data, so stall INs and OUTs */
    PD_USBFS_ep0Mode =  PD_USBFS_MODE_STALL_IN_OUT;
}


/*******************************************************************************
* Function Name: PD_USBFS_InitControlWrite
********************************************************************************
*
* Summary:
*  Initialize a control write transaction
*
* Parameters:
*  None.
*
* Return:
*  requestHandled state.
*
* Global variables:
*  PD_USBFS_USBFS_transferState - set to TRANS_STATE_CONTROL_WRITE
*  PD_USBFS_ep0Toggle - set to EP0_CNT_DATA_TOGGLE
*  PD_USBFS_ep0Mode  - set to MODE_ACK_OUT_STATUS_IN
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 PD_USBFS_InitControlWrite(void) 
{
    uint16 xferCount;

    /* Set up the state machine */
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_CONTROL_WRITE;
    /* This might not be necessary */
    PD_USBFS_ep0Toggle = PD_USBFS_EP0_CNT_DATA_TOGGLE;
    /* Initialize the Status Block */
    PD_USBFS_InitializeStatusBlock();

    xferCount = (((uint16)CY_GET_REG8(PD_USBFS_lengthHi) << 8u) | (CY_GET_REG8(PD_USBFS_lengthLo)));

    if (PD_USBFS_currentTD.count > xferCount)
    {
        PD_USBFS_currentTD.count = xferCount;
    }

    /* Expect Data or Status Stage */
    PD_USBFS_ep0Mode = PD_USBFS_MODE_ACK_OUT_STATUS_IN;

    return(PD_USBFS_TRUE);
}


/*******************************************************************************
* Function Name: PD_USBFS_ControlWriteDataStage
********************************************************************************
*
* Summary:
*  Handle the Data Stage of a control write transfer
*       1. Get the data (We assume the destination was validated previously)
*       2. Update the count and data toggle
*       3. Update the mode register for the next transaction
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_transferByteCount - Update the transfer byte count from the
*    last transaction.
*  PD_USBFS_ep0Count - counts the data loaded from the SIE memory
*    in current packet.
*  PD_USBFS_transferByteCount - sum of the previous bytes transferred
*    on previous packets(sum of USBFS_lastPacketSize)
*  PD_USBFS_ep0Toggle - inverted
*  PD_USBFS_ep0Mode  - set to MODE_ACK_OUT_STATUS_IN.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_ControlWriteDataStage(void) 
{
    uint8 ep0Count;
    uint8 regIndex = 0u;

    ep0Count = (CY_GET_REG8(PD_USBFS_EP0_CNT_PTR) & PD_USBFS_EPX_CNT0_MASK) -
               PD_USBFS_EPX_CNTX_CRC_COUNT;

    PD_USBFS_transferByteCount += ep0Count;

    while ((PD_USBFS_currentTD.count > 0u) && (ep0Count > 0u))
    {
        *PD_USBFS_currentTD.pData = CY_GET_REG8((reg8 *)(PD_USBFS_EP0_DR0_IND + regIndex));
        PD_USBFS_currentTD.pData = &PD_USBFS_currentTD.pData[1u];
        regIndex++;
        ep0Count--;
        PD_USBFS_currentTD.count--;
    }
    PD_USBFS_ep0Count = ep0Count;
    /* Update the data toggle */
    PD_USBFS_ep0Toggle ^= PD_USBFS_EP0_CNT_DATA_TOGGLE;
    /* Expect Data or Status Stage */
    PD_USBFS_ep0Mode = PD_USBFS_MODE_ACK_OUT_STATUS_IN;
}


/*******************************************************************************
* Function Name: PD_USBFS_ControlWriteStatusStage
********************************************************************************
*
* Summary:
*  Handle the Status Stage of a control write transfer
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_transferState - set to TRANS_STATE_IDLE.
*  PD_USBFS_USBFS_ep0Mode  - set to MODE_STALL_IN_OUT.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_ControlWriteStatusStage(void) 
{
    /* Go Idle */
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_IDLE;
    /* Update the completion block */
    PD_USBFS_UpdateStatusBlock(PD_USBFS_XFER_STATUS_ACK);
    /* We expect no more data, so stall INs and OUTs */
    PD_USBFS_ep0Mode = PD_USBFS_MODE_STALL_IN_OUT;
}


/*******************************************************************************
* Function Name: PD_USBFS_InitNoDataControlTransfer
********************************************************************************
*
* Summary:
*  Initialize a no data control transfer
*
* Parameters:
*  None.
*
* Return:
*  requestHandled state.
*
* Global variables:
*  PD_USBFS_transferState - set to TRANS_STATE_NO_DATA_CONTROL.
*  PD_USBFS_ep0Mode  - set to MODE_STATUS_IN_ONLY.
*  PD_USBFS_ep0Count - cleared.
*  PD_USBFS_ep0Toggle - set to EP0_CNT_DATA_TOGGLE
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 PD_USBFS_InitNoDataControlTransfer(void) 
{
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_NO_DATA_CONTROL;
    PD_USBFS_ep0Mode = PD_USBFS_MODE_STATUS_IN_ONLY;
    PD_USBFS_ep0Toggle = PD_USBFS_EP0_CNT_DATA_TOGGLE;
    PD_USBFS_ep0Count = 0u;

    return(PD_USBFS_TRUE);
}


/*******************************************************************************
* Function Name: PD_USBFS_NoDataControlStatusStage
********************************************************************************
* Summary:
*  Handle the Status Stage of a no data control transfer.
*
*  SET_ADDRESS is special, since we need to receive the status stage with
*  the old address.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_transferState - set to TRANS_STATE_IDLE.
*  PD_USBFS_ep0Mode  - set to MODE_STALL_IN_OUT.
*  PD_USBFS_ep0Toggle - set to EP0_CNT_DATA_TOGGLE
*  PD_USBFS_deviceAddress - used to set new address and cleared
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_NoDataControlStatusStage(void) 
{
    /* Change the USB address register if we got a SET_ADDRESS. */
    if (PD_USBFS_deviceAddress != 0u)
    {
        CY_SET_REG8(PD_USBFS_CR0_PTR, PD_USBFS_deviceAddress | PD_USBFS_CR0_ENABLE);
        PD_USBFS_deviceAddress = 0u;
    }
    /* Go Idle */
    PD_USBFS_transferState = PD_USBFS_TRANS_STATE_IDLE;
    /* Update the completion block */
    PD_USBFS_UpdateStatusBlock(PD_USBFS_XFER_STATUS_ACK);
     /* We expect no more data, so stall INs and OUTs */
    PD_USBFS_ep0Mode = PD_USBFS_MODE_STALL_IN_OUT;
}


/*******************************************************************************
* Function Name: PD_USBFS_UpdateStatusBlock
********************************************************************************
*
* Summary:
*  Update the Completion Status Block for a Request.  The block is updated
*  with the completion code the PD_USBFS_transferByteCount.  The
*  StatusBlock Pointer is set to NULL.
*
* Parameters:
*  completionCode - status.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_currentTD.pStatusBlock->status - updated by the
*    completionCode parameter.
*  PD_USBFS_currentTD.pStatusBlock->length - updated.
*  PD_USBFS_currentTD.pStatusBlock - cleared.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_UpdateStatusBlock(uint8 completionCode) 
{
    if (PD_USBFS_currentTD.pStatusBlock != NULL)
    {
        PD_USBFS_currentTD.pStatusBlock->status = completionCode;
        PD_USBFS_currentTD.pStatusBlock->length = PD_USBFS_transferByteCount;
        PD_USBFS_currentTD.pStatusBlock = NULL;
    }
}


/*******************************************************************************
* Function Name: PD_USBFS_InitializeStatusBlock
********************************************************************************
*
* Summary:
*  Initialize the Completion Status Block for a Request.  The completion
*  code is set to USB_XFER_IDLE.
*
*  Also, initializes PD_USBFS_transferByteCount.  Save some space,
*  this is the only consumer.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  PD_USBFS_currentTD.pStatusBlock->status - set to XFER_IDLE.
*  PD_USBFS_currentTD.pStatusBlock->length - cleared.
*  PD_USBFS_transferByteCount - cleared.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void PD_USBFS_InitializeStatusBlock(void) 
{
    PD_USBFS_transferByteCount = 0u;
    if(PD_USBFS_currentTD.pStatusBlock != NULL)
    {
        PD_USBFS_currentTD.pStatusBlock->status = PD_USBFS_XFER_IDLE;
        PD_USBFS_currentTD.pStatusBlock->length = 0u;
    }
}


/* [] END OF FILE */
