//=====================================================================
//
//	PD_j1XML.c - j1sys xml
//
//		simple xml reader, parser, writer
//
//		also includes a command buffer parser as a psuedo
//		xml element
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

//#include "main.h"

#include <project.h>
//#include "PD_j1XML.h"

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

//---------------------------------------------------------------------
//
//	private data area
//
//---------------------------------------------------------------------

j1XML_IOMODE	__j1XML_IOMODE;
uint32			__j1XML_baseOffset;
uint32			__j1XML_fileSize;

#define	ISSPACE(x)	isspace((uint8) x)
#define	ISALPHA(x)	isalpha((uint8) x)
#define	ISALNUM(x)	isalnum((uint8) x)
#define	ISXDIGIT(x)	isxdigit((uint8) x)
#define	TOLOWER(x)	tolower((uint8) x)
#define	TOUPPER(x)	toupper((uint8) x)


//---------------------------------------------------------------------
//
//	j1XML_Open() - simple 'open' file
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_Open(j1XML_IOMODE ioMode, uint32 baseOffset, uint32 fileSize)
{
	__j1XML_IOMODE		= ioMode;
	__j1XML_baseOffset	= baseOffset;
	__j1XML_fileSize	= fileSize;

	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_Close() - simple 'close' file
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_Close()
{
	__j1XML_IOMODE = j1XML_IOMODE_CLOSED;
	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_ReadChar() - simple reader
//
//---------------------------------------------------------------------

bool j1XML_ReadChar(uint32 *offset, char *value, bool skipWhite)
{
	char	c;
	bool	white = true;
	uint32	thisOffset = *offset;

	while (white)
	{
		if (thisOffset >= __j1XML_fileSize) return false;

		switch (__j1XML_IOMODE)
		{
		case j1XML_IOMODE_READ_MEM:
			c = *(char *) (__j1XML_baseOffset + thisOffset);
			break;
#if 0
		case j1XML_IOMODE_READ_EXTFLASH:
			_ReadArray(, __j1XML_baseOffset + thisOffset, (BYTE *) &c, 1);
			break;
#endif
		default:
			c = 0;
		}

		if (c == 0 || c == 0xff) return false;
		if (!skipWhite) break;
		white = ISSPACE(c);
		if (white) thisOffset++;
	}

	*offset = thisOffset;
	*value = c;
	return true;
}

//---------------------------------------------------------------------
//
//	j1XML_ReadElement() - get the next complete element
//
//		params:
//			offset		- starting offset value
//			element		- pointer to a j1XML_ELEMENT structure
//
//			WARNING - read the notes about the element structure
//			in j1XML.h!!!
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_ReadElement(uint32 offset, j1XML_ELEMENT *element, bool cmdParse)
{
	char	c;
	bool	startFound = false;
	bool	textFound = false;
	int		fieldIdx, fieldSize, aIdx;
	char	*fieldPtr;
	bool	whiteAllowed;
	char	quote;
	
	quote = 0;


	// initialize text start/end offset to zero
	element->tStartOffset = 0;
	element->tEndOffset = 0;

	// set the start offset so they know where error starts
	element->eStartOffset = offset;
	element->eEndOffset = 0;

	// init the end-of-string in eNameBfr for safety
	element->eNameBfr[0] = 0;

	// init the end-of-string(s) in aBfr for safety
	fieldPtr = element->aBfr;
	for (aIdx = 0; aIdx < element->aCount; aIdx++)
	{
		*fieldPtr = 0;
		fieldPtr += element->aNameSize;
		*fieldPtr = 0;
		fieldPtr += element->aValueSize;
	}

	if (!cmdParse)
	{
		// find the starting '<'
		while (j1XML_ReadChar(&offset, &c, true))
		{
			if (c == '<')
			{
				startFound = true;
				break;
			}

			if (!textFound)
			{
				element->tStartOffset = offset;
				textFound = true;
			}

			element->tEndOffset = offset;
			offset++;
		}

		// if not found and no text found then EOF
		if (!startFound)
		{
			if (!textFound) return j1XML_RESULT_READ_EOF;
			// textFound - ERROR text after last element
			return j1XML_RESULT_READ_PARSE_ERROR_TEXT_AFTER;
		}

		// store the real start offset and move to next character
		element->eStartOffset = offset;
		offset++;
	}

	// read next char? no - ERROR
	if (!j1XML_ReadChar(&offset, &c, false)) return j1XML_RESULT_READ_PARSE_ERROR_OPEN_IS_LAST;

	if (cmdParse) element->eType = j1XML_ETYPE_CMD;
	else
	{
		// check for element type
		switch (c)
		{
		case '?':
			element->eType = j1XML_ETYPE_DECL;
			offset++;
			break;

		case '!':
			element->eType = j1XML_ETYPE_COMM;
			offset++;
			break;

		case '/':
			element->eType = j1XML_ETYPE_STOP;
			offset++;
			break;

		default:
			element->eType = j1XML_ETYPE_START;
		}
	}

	//----------------------------------
	// handle comments - store in aBfr
	//----------------------------------
	if (element->eType == j1XML_ETYPE_COMM)
	{
		fieldIdx	= 0;
		fieldSize	= element->aCount * (element->aNameSize + element->aValueSize);
		fieldPtr	= element->aBfr;

		while (j1XML_ReadChar(&offset, &c, true))
		{
			if (c == '>') break;

			if  (fieldIdx < fieldSize - 1)
			{
				*fieldPtr++ = c;
				fieldIdx++;
			}
			else return j1XML_RESULT_READ_PARSE_ERROR_COMMENT_TOO_BIG;

			offset++;
		}

		if (c == '>')
		{
			*fieldPtr = 0;
			element->eEndOffset = offset;
			return j1XML_RESULT_OK;
		}

		return j1XML_RESULT_READ_PARSE_ERROR_OPEN_COMMENT;
	} //--- end of comment handler ---

	// for all non-comment now parse element name
	fieldIdx = 0;
	fieldPtr = element->eNameBfr;

	while (j1XML_ReadChar(&offset, &c, false))
	{
		// first character has to be alpha
		if (fieldIdx == 0) if (!ISALPHA(c))
		{
			if (!cmdParse) return j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_FIRST_NOT_ALPHA;
					  else return j1XML_RESULT_READ_PARSE_ERROR_COMMAND_FIRST_NOT_ALPHA;
		}

		if (ISALNUM(c) || c == '_')
		{
			if (fieldIdx < element->eNameSize - 1)
			{
#if j1XML_PARAM_LOWER_CASE_NAMES
				c = TOLOWER(c);
#endif
				*fieldPtr++ = c;
				fieldIdx++;
				offset++;
				c = 0;
				continue;
			}
			// name too large
			else
			{
				if (!cmdParse) return j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_TOO_BIG;
						  else return j1XML_RESULT_READ_PARSE_ERROR_COMMAND_TOO_BIG;
			}
		}
			
		if (ISSPACE(c)) break;
		if (!cmdParse) if (c == '?' || c == '/' || c == '>') break;

		// invalid character in name
		if (!cmdParse) return j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_NOT_ALPHANUMERIC;
				  else return j1XML_RESULT_READ_PARSE_ERROR_COMMAND_NOT_ALPHANUMERIC;
	}

	// empty name not allowed
	if (fieldIdx == 0)
	{
		if (!cmdParse) return j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_EMPTY;
				  else return j1XML_RESULT_READ_PARSE_ERROR_COMMAND_EMPTY;
	}

	// terminate the string
	*fieldPtr = 0;

	// terminator after element name
	if (cmdParse)
	{
		if (c == 0) goto parseEnd;
	}
	else
	{
		if (c == '?' || c == '/' || c == '>') goto parseEnd;
	}

	// now parse attributes
	aIdx = 0;

	// parse the name/value pairs
	while (true)
	{
		// parse name
		fieldIdx = 0;
		fieldSize = element->aNameSize;
		fieldPtr = element->aBfr + aIdx * (element->aNameSize + element->aValueSize);

		whiteAllowed = true;	// allow to skip 'before' whitespace
		if (cmdParse) c = 0;

		while (j1XML_ReadChar(&offset, &c, whiteAllowed))
		{
			whiteAllowed = false;	// shutdown whiteAllowed

#if j1XML_PARAM_ALLOW_BARE_WORD
			if (!cmdParse) if (c == '?' || c == '/' || c == '>') break;
			if (c == ' ') break;
#endif

			if (fieldIdx == 0)
			{
				// too many attributes?
				if (aIdx >= element->aCount) return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_TOO_MANY;

				// first character has to be alpha
				if (!ISALPHA(c)) return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_FIRST_NOT_ALPHA;
			}

			if (ISALNUM(c) || c == '_')
			{
				if (fieldIdx < fieldSize - 1)
				{
#if j1XML_PARAM_LOWER_CASE_NAMES
					c = TOLOWER(c);
#endif
					*fieldPtr++ = c;
					fieldIdx++;
					offset++;
					c = 0;
					continue;
				}
				// name too large
				else return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_TOO_BIG;
			}
			
			if (c == '=') break;

			// invalid character in name
			return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_NOT_ALPHANUMERIC;
		}

		// terminate the string
		*fieldPtr = 0;

		if (cmdParse)
		{
			if (c == 0) break;
		}
		else
		{
			if (c == '?' || c == '/' || c == '>') break;
		}

		// empty attribute name not allowed
		if (fieldIdx == 0) return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_EMPTY;

		if (c == ' ')
		{
			aIdx++;
			continue;
		}

		// skip past the '='
		offset++;

		// parse value
		fieldIdx = -1;
		fieldSize = element->aValueSize;
		fieldPtr = element->aBfr + aIdx * (element->aNameSize + element->aValueSize) + element->aNameSize;
		
		while (j1XML_ReadChar(&offset, &c, false))
		{
			if (fieldIdx == -1)
			{
				// error if not single or double quotes
				if (c != '\'' && c != '"')
				{
					if (!cmdParse) return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_INVALID_QUOTE;
					quote = ' ';
					*fieldPtr++ = c;
					fieldIdx++;
				}		
				else quote = c;

				offset++;
				fieldIdx++;
				continue;
			}

			if (c == quote)
			{
				if (c != ' ') offset++;
				break;
			}

			if (fieldIdx < fieldSize - 1)
			{
				*fieldPtr++ = c;
				fieldIdx++;
				offset++;
				continue;
			}
			// name too large
			else return j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_TOO_BIG;
		}

		// terminate the string
		*fieldPtr = 0;

		// bump for next attribute pair
		aIdx++;
	}

parseEnd:
	if (c == '?')
	{
		// must match a decl
		if (!element->eType == j1XML_ETYPE_DECL) return j1XML_RESULT_READ_PARSE_ERROR_DECL_END_MISMATCH;
		offset++;
		if (!j1XML_ReadChar(&offset, &c, false)) return j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_DECL_END;
		if (c != '>') return j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_DECL_END;
	}

	else if (c == '/')
	{
		// must think it's a start
		if (!element->eType == j1XML_ETYPE_START) return j1XML_RESULT_READ_PARSE_ERROR_EMPTY_END_MISMATCH;
		element->eType = j1XML_ETYPE_EMPTY;
		offset++;
		if (!j1XML_ReadChar(&offset, &c, false)) return j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_EMPTY_END;
		if (c != '>') return j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_EMPTY_END;
	}

	else if (c == '>')
	{
		switch (element->eType)
		{
		case j1XML_ETYPE_DECL:
			return j1XML_RESULT_READ_PARSE_ERROR_DECL_MISSING_END;
		case j1XML_ETYPE_COMM:
			return j1XML_RESULT_READ_PARSE_ERROR_COMM_LOGIC;
		case j1XML_ETYPE_START:
		case j1XML_ETYPE_STOP:
			break;
		case j1XML_ETYPE_EMPTY:
			return j1XML_RESULT_READ_PARSE_ERROR_EMPTY_LOGIC;
		default:
			return j1XML_RESULT_READ_PARSE_ERROR_LOGIC;
		}
	}

	element->eEndOffset = offset;
	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_WriteChar() - write a character to the flash
//
//---------------------------------------------------------------------

bool j1XML_WriteChar(uint32 *offset, char value)
{
	if (*offset > __j1XML_fileSize - 1) return false;

#if 0
	if (__j1XML_IOMODE == j1XML_IOMODE_WRITE_EXTFLASH)
	{
		if (*offset == 0) _BeginWrite(, __j1XML_baseOffset);
		_Write(, value);
		(*offset)++;
	}
#endif

	if (__j1XML_IOMODE == j1XML_IOMODE_WRITE_MEM)
	{
		*(char *) (__j1XML_baseOffset + *offset) = value;
		(*offset)++;
	}
	
	return true;
}

//---------------------------------------------------------------------
//
//	j1XML_WriteString() - write a string to the flash
//
//---------------------------------------------------------------------

bool j1XML_WriteString(uint32 *offset, char *value)
{
	int	len;

	len = strlen(value);

	if (*offset > __j1XML_fileSize - len) return false;

#if 0
	if (__j1XML_IOMODE == j1XML_IOMODE_WRITE_EXTFLASH)
	{
		if (*offset == 0) _BeginWrite(, __j1XML_baseOffset);
		_WriteArray(, (BYTE *) value, len);
		(*offset) += len;
	}
#endif

	if (__j1XML_IOMODE == j1XML_IOMODE_WRITE_MEM)
	{
		while (*value)
		{
			*(char *) (__j1XML_baseOffset + *offset) = *value++;
			(*offset)++;
		}
	}

	return true;
}

//---------------------------------------------------------------------
//
//	j1XMLWrite_Element() - write an element to 'file'
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_WriteElement(uint32 offset, uint8 inset, j1XML_ELEMENT *element, uint32 *nextOffset)
{
	int		aIdx;
//	int		fieldIdx;
//	int		fieldSize;
	char	*fieldPtr;
//	char	c;
	char	quote;

	// do inset if user supplies
	while (inset--)
	{
		if (!j1XML_WriteChar(&offset, ' ')) return j1XML_RESULT_WRITE_NO_ROOM;
	}

	switch (element->eType)
	{
/***
	case j1XML_ETYPE_DECL:
		// open angle
		if (!j1XML_WriteChar(&offset, '<')) return j1XML_RESULT_WRITE_NO_ROOM;
		// decl character '?'
		if (!j1XML_WriteChar(&offset, '?')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	case j1XML_ETYPE_COMM:
		// open angle
		if (!j1XML_WriteChar(&offset, '<')) return j1XML_RESULT_WRITE_NO_ROOM;
		// comm character '!'
		if (!j1XML_WriteChar(&offset, '!')) return j1XML_RESULT_WRITE_NO_ROOM;

		fieldIdx = 0;
		fieldSize = element->aCount * (element->aNameSize + element->aValueSize);
		fieldPtr = element->aBfr;

		while (fieldIdx < fieldSize)
		{
			c = *fieldPtr++;
			if (c == 0) break;
			if (!j1XML_WriteChar(&offset, c)) return j1XML_RESULT_WRITE_NO_ROOM;
			fieldIdx++;
		}

		// close angle
		if (!j1XML_WriteChar(&offset, '>')) return j1XML_RESULT_WRITE_NO_ROOM;
		// crlf
		if (!j1XML_WriteChar(&offset, '\r')) return j1XML_RESULT_WRITE_NO_ROOM;
		if (!j1XML_WriteChar(&offset, '\n')) return j1XML_RESULT_WRITE_NO_ROOM;

		*nextOffset = offset;

		return j1XML_RESULT_OK;
		break;
***/		
	case j1XML_ETYPE_START:
	case j1XML_ETYPE_EMPTY:
		// open angle
		if (!j1XML_WriteChar(&offset, '<')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	case j1XML_ETYPE_STOP:
		// open angle
		if (!j1XML_WriteChar(&offset, '<')) return j1XML_RESULT_WRITE_NO_ROOM;
		// stop char '/'
		if (!j1XML_WriteChar(&offset, '/')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	default:
		return j1XML_RESULT_WRITE_UNKNOWN_ETYPE;
	}

	// now do the element name
//	fieldSize = element->eNameSize;
	fieldPtr = element->eNameBfr;
	if (!j1XML_WriteString(&offset, fieldPtr)) return j1XML_RESULT_WRITE_NO_ROOM;

	aIdx = 0;

	while (aIdx < element->aCount)
	{
		// setup for the attribute name
//		fieldIdx	= 0;
//		fieldSize	= element->aNameSize;
		fieldPtr	= element->aBfr + aIdx * (element->aNameSize + element->aValueSize);

		// only do if there is an attribute name
		if (*fieldPtr)
		{
			// first the whitespace
			if (!j1XML_WriteChar(&offset, ' ')) return j1XML_RESULT_WRITE_NO_ROOM;

			// now the attribute name
			if (!j1XML_WriteString(&offset, fieldPtr)) return j1XML_RESULT_WRITE_NO_ROOM;

			// setup for the attribute value
//			fieldIdx	= 0;
//			fieldSize	= element->aNameSize;
			fieldPtr	= element->aBfr + aIdx * (element->aNameSize + element->aValueSize) + element->aNameSize;

#if j1XML_PARAM_ALLOW_BARE_WORD
			// bareword if
			if (fieldPtr)		// kludge!!!!!! - fix it
			{
#endif
				if (!j1XML_WriteChar(&offset, '=')) return j1XML_RESULT_WRITE_NO_ROOM;
				
				quote = 0;
				if (strchr(fieldPtr, '"' )) quote = '\'';
				if (strchr(fieldPtr, '\''))
				{
					if (quote) return j1XML_RESULT_WRITE_NO_POSSIBLE_QUOTE;
					quote = '"';
				}
				if (quote == 0) quote = '\'';

				// write the opening quote
				if (!j1XML_WriteChar(&offset, quote)) return j1XML_RESULT_WRITE_NO_ROOM;

				// now the value
				if (*fieldPtr) if (!j1XML_WriteString(&offset, fieldPtr)) return j1XML_RESULT_WRITE_NO_ROOM;

				// write the closing quote
				if (!j1XML_WriteChar(&offset, quote)) return j1XML_RESULT_WRITE_NO_ROOM;

#if j1XML_PARAM_ALLOW_BARE_WORD
			}
#endif
		}
		
		aIdx++;
	}

	switch (element->eType)
	{
	case j1XML_ETYPE_DECL:
		if (!j1XML_WriteChar(&offset, '?')) return j1XML_RESULT_WRITE_NO_ROOM;
		if (!j1XML_WriteChar(&offset, '>')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	case j1XML_ETYPE_START:
	case j1XML_ETYPE_STOP:
		if (!j1XML_WriteChar(&offset, '>')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	case j1XML_ETYPE_EMPTY:
		if (!j1XML_WriteChar(&offset, '/')) return j1XML_RESULT_WRITE_NO_ROOM;
		if (!j1XML_WriteChar(&offset, '>')) return j1XML_RESULT_WRITE_NO_ROOM;
		break;

	default:
		return j1XML_RESULT_WRITE_ETYPE_LOGIC;
	}

	// crlf
	if (!j1XML_WriteChar(&offset, '\r')) return j1XML_RESULT_WRITE_NO_ROOM;
	if (!j1XML_WriteChar(&offset, '\n')) return j1XML_RESULT_WRITE_NO_ROOM;

	*nextOffset = offset;

	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_ParseElement() - parse the element against a dictionary
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_ParseElement(j1XML_ELEMENT *element, int eTbl_Size, const j1XML_DICT_ELEM const * const *eTbl, int *depth, int maxDepth, uint16 *eStack, bool cmdParse)
{
	int		elemIdx, element_aIdx, elem_aIdx, i;
	char	*namPtr, *valPtr, *termPtr;
	int		namLen, valLen;

	j1XML_DICT_ELEM	const * elem;
	j1XML_DICT_ATTR	const * const * aTbl;
	j1XML_DICT_ATTR	const * attr;
	
	elem = NULL;
	attr = NULL;

	// init the attr id table
	if (element->aIDTbl)
	{
		for (i = 0; i < element->aCount; i++)
		{
			element->aIDTbl[i] = 0;
		}
	}

	// init the attr value table
	if (element->vTbl)
	{
		for (i = 0; i < element->aCount; i++)
		{
			element->vTbl[i] = 0;
		}
	}

	// scan the elem dictionary to find a match
	for (elemIdx = 0; elemIdx < eTbl_Size; elemIdx++)
	{
		elem = eTbl[elemIdx];

		// if name matches, exit the loop
		if (strcmp(element->eNameBfr, elem->name) == 0) break;
	}

	// did we look at all of them, yes - declare error
	if (elemIdx == eTbl_Size)
	{
		if (!cmdParse) return j1XML_RESULT_PARSE_ELEM_INVALID;
			  	  else return j1XML_RESULT_PARSE_CMD_INVALID;
	}

	// set the eID in the element
	element->eID = elem->elemID;

	// get the attr dictionary table
	aTbl = elem->aTbl;

	// loop through all the attributes in the element
	for (element_aIdx = 0; element_aIdx < element->aCount; element_aIdx++)
	{
		// set name/val pointers and string length
		namPtr = element->aBfr + (element_aIdx * (element->aNameSize + element->aValueSize));
		valPtr = namPtr + element->aNameSize;
		namLen = strlen(namPtr);
		valLen = strlen(valPtr);

		// if empty attr name then skip it
		if (namLen == 0) continue;

		// if 'null' attribute table can't have attributes
		if (aTbl == NULL) return j1XML_RESULT_PARSE_ATTR_INVALID;

		// scan through attr dictionary to find a match
		for (elem_aIdx = 0; elem_aIdx < elem->aCnt; elem_aIdx++)
		{
			attr = aTbl[elem_aIdx];

			// if name matches then exit the loop
			if (strcmp(namPtr, attr->name) == 0) break;
		}

		// did we look at all of them, yes - declare error
		if (elem_aIdx == elem->aCnt)
		{
			return j1XML_RESULT_PARSE_ATTR_INVALID;
		}

		// if attr id table (should be) then scan and store
		if (element->aIDTbl)
		{
			// check for duplicate attribute
			for (i = 0; i < element->aCount; i++)
			{
				if (attr->attrID == element->aIDTbl[i]) break;
			}

			// did we quit early, yes - declare error
			if (i < element->aCount)
			{
				return j1XML_RESULT_PARSE_ATTR_DUPE;
			}

			element->aIDTbl[element_aIdx] = attr->attrID;
		}

		// check the value string length against minLen
		if (valLen < attr->minLen)
		{
			return j1XML_RESULT_PARSE_ATTR_MINLEN;
		}

		// check the value string length against maxLen
		if (valLen > attr->maxLen)
		{
			return j1XML_RESULT_PARSE_ATTR_MAXLEN;
		}

		// now handle the value conversion and validation
		switch (attr->aType)
		{
		case j1XML_DICT_ATYPE_BARE:
			// bare word type should have zero valLen
			break;

		case j1XML_DICT_ATYPE_STRING:
			// if we have entries in a validation table
			if (attr->vCnt && attr->vTbl)
			{
				const char	const * const *_vTbl = attr->vTbl;

				// scan the table
				for (i = 0; i < attr->vCnt; i++)
				{
					// if matches quit early
					if (strcmp(valPtr, _vTbl[i]) == 0) break;
				}

				// did we look at all of them - yes, error
				if (i == attr->vCnt)
				{
					return j1XML_RESULT_PARSE_ATTR_STRING_INVALID;
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save 1 based index into valid entries
					element->vTbl[element_aIdx] = i + 1;
				}
			}
			// otherwise just save the pointer to the text if we have a vTbl
			else if (element->vTbl)
			{
				element->vTbl[element_aIdx] = (j1XML_E_VAL) valPtr;
			}
			break;

		case j1XML_DICT_ATYPE_CHAR:
			// if we have entries in a validation table
			if (attr->vCnt && attr->vTbl)
			{
				const char	const * const *_vTbl = attr->vTbl;

				// scan the table
				for (i = 0; i < attr->vCnt; i++)
				{
					// if matches quit early
					if (strcmp(valPtr, _vTbl[i]) == 0) break;
				}

				// did we look at all of them - yes, error
				if (i == attr->vCnt)
				{
					return j1XML_RESULT_PARSE_ATTR_STRING_INVALID;
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save 1 based index into valid entries
					element->vTbl[element_aIdx] = i + 1;
				}
			}
			// otherwise just save the pointer to the text if we have a vTbl
			else if (element->vTbl)
			{
				element->vTbl[element_aIdx] = *valPtr;
			}
			break;

		case j1XML_DICT_ATYPE_BYTE:
			{
				uint8 byteVal;

				// try the conversion
				byteVal = (uint8) j1_strtoul(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_BYTE_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = byteVal;
				}
			}
			break;

		case j1XML_DICT_ATYPE_HEXBYTE:
			{
				uint8 byteVal;

				// try the conversion
				byteVal = (uint8) j1_strtoul(valPtr, &termPtr, 16);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_BYTE_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = byteVal;
				}
			}
			break;

		case j1XML_DICT_ATYPE_WORD:
			{
				uint16 wordVal;

				// try the conversion
				wordVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_WORD_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = wordVal;
				}
			}
			break;

		case j1XML_DICT_ATYPE_DWORD:
			{
				uint32 dwordVal;

				// try the conversion
				dwordVal = j1_strtoul(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_WORD_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = dwordVal;
				}
			}
			break;

		case j1XML_DICT_ATYPE_INT16:
			{
				int16 int16Val;

				// try the conversion
				int16Val = (int16) strtol(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_INT16_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = int16Val;
				}
			}
			break;

		case j1XML_DICT_ATYPE_INT32:
			{
				int32 int32Val;

				// try the conversion
				int32Val = strtol(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_INT32_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = int32Val;
				}
			}
			break;

		case j1XML_DICT_ATYPE_BOOL:
			{
				uint8 boolVal;

				// try the conversion
				boolVal = (uint8) j1_strtoul(valPtr, &termPtr, 0);

				// if we point to a non-zero we have error
				if (*termPtr)
				{
					return j1XML_RESULT_PARSE_ATTR_BOOL_INVALID;
				}

				if (boolVal != 0 && boolVal != 1)
				{
					return j1XML_RESULT_PARSE_ATTR_BOOL_INVALID;
				}

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = boolVal;
				}
			}
			break;

		case j1XML_DICT_ATYPE_IP:
			{
				uint16			octetVal;
				j1UINT32_VAL	ip;

				// try 1st octet
				octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
				// if we point to not '.' we have error
				if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				ip.B[0] = octetVal;
				valPtr = ++termPtr;

				// try 2nd octet
				octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
				// if we point to not '.' we have error
				if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				ip.B[1] = octetVal;
				valPtr = ++termPtr;

				// try 3rd octet
				octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
				// if we point to not '.' we have error
				if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				ip.B[2] = octetVal;
				valPtr = ++termPtr;

				// try 4th octet
				octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
				// if we don't point to '\0' we have error
				if (*termPtr) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
				ip.B[3] = octetVal;

				// do we have a validation table?
				if (attr->vCnt == 2 && attr->vTbl)
				{
				}

				// do we have a vTbl to store into?
				if (element->vTbl)
				{
					// save the value
					element->vTbl[element_aIdx] = ip.Val;
				}
			}
			break;

		case j1XML_DICT_ATYPE_MAC:
			{
//				uint16		octetVal;
				int			i;
				char		hi, lo, term;

				if (strlen(valPtr) != 17) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;

				for (i = 0; i < 6; i++)
				{
					hi   = *valPtr++;
					lo   = *valPtr++;
					term = *valPtr++;

					if (!ISXDIGIT(hi)) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
					if (!ISXDIGIT(lo)) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;

					if (i < 5)
					{
						if (term != ':') return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
					}
					else
					{
						if (term != 0x00) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
					}
				}
			}
			break;

		default:
			return j1XML_RESULT_PARSE_ATTR_ATYPE;
		}
	}

	// now process the eStack depth

	switch(element->eType)
	{
	case j1XML_ETYPE_START:
		if (*depth == maxDepth)
		{
			return j1XML_RESULT_PARSE_ESTACK_EXCEEDED;
		}

		eStack[*depth] = element->eID;
		(*depth)++;
		break;
		
	case j1XML_ETYPE_STOP:
		if (*depth == 0)
		{
			return j1XML_RESULT_PARSE_ESTACK_EXHAUSTED;
		}

		if (eStack[*depth - 1] != element->eID)
		{
			return j1XML_RESULT_PARSE_ESTACK_MISMATCH;
		}

		(*depth)--;
		break;

	case j1XML_ETYPE_EMPTY:
	case j1XML_ETYPE_CMD:
		break;

	default:
		return j1XML_RESULT_PARSE_ETYPE_ERROR;
	}

	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_eTypeDecode() - return a string for each eType
//
//---------------------------------------------------------------------

char *j1XML_eTypeDecode(j1XML_ETYPE eType)
{
	switch (eType)
	{
	case j1XML_ETYPE_DECL:
		return "declaration";
	case j1XML_ETYPE_COMM:
		return "comment";
	case j1XML_ETYPE_START:
		return "start";
	case j1XML_ETYPE_STOP:
		return "stop";
	case j1XML_ETYPE_EMPTY:
		return "empty";
	case j1XML_ETYPE_CMD:
		return "cmd";
	}

	return "unknown element type";
}

//---------------------------------------------------------------------
//
//	j1XML_ErrorDecode() - return a string for an error result
//
//---------------------------------------------------------------------

char *j1XML_ErrorDecode(j1XML_RESULT result)
{
	switch (result)
	{
	case j1XML_RESULT_OK:
		return "OK";

	case j1XML_RESULT_OPEN_ERROR:
		return "'file' open error";
	case j1XML_RESULT_CLOSE_ERROR:
		return "'file' close error";

	case j1XML_RESULT_READ_EOF:
		return "end-of-file";
	case j1XML_RESULT_READ_NOT_OPENED_CORRECTLY:
		return "'file' not opened for read";
	case j1XML_RESULT_READ_ERROR:
		return "read error";

	case j1XML_RESULT_READ_PARSE_ERROR_TEXT_AFTER:
		return "ends with text after element";
	case j1XML_RESULT_READ_PARSE_ERROR_OPEN_IS_LAST:
		return "open angle is last character";
	case j1XML_RESULT_READ_PARSE_ERROR_COMMENT_TOO_BIG:
		return "comment too big";
	case j1XML_RESULT_READ_PARSE_ERROR_OPEN_COMMENT:
		return "open ended comment";

	case j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_FIRST_NOT_ALPHA:
		return "element name: first character not alpha";
	case j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_TOO_BIG:
		return "element name: too big";
	case j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_NOT_ALPHANUMERIC:
		return "element name: not alphanumeric";
	case j1XML_RESULT_READ_PARSE_ERROR_ELEMENT_NAME_EMPTY:
		return "element name: empty";

	case j1XML_RESULT_READ_PARSE_ERROR_COMMAND_FIRST_NOT_ALPHA:
		return "command: first character not alpha";
	case j1XML_RESULT_READ_PARSE_ERROR_COMMAND_TOO_BIG:
		return "command: too big";
	case j1XML_RESULT_READ_PARSE_ERROR_COMMAND_NOT_ALPHANUMERIC:
		return "command: not alphanumeric";
	case j1XML_RESULT_READ_PARSE_ERROR_COMMAND_EMPTY:
		return "command: empty";

	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_TOO_MANY:
		return "too many attributes";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_FIRST_NOT_ALPHA:
		return "attribute name: first character not alpha";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_TOO_BIG:
		return "attribute name: too big";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_NOT_ALPHANUMERIC:
		return "attribute name: not alphanumeric";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_NAME_EMPTY:
		return "attribute name: empty";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_INVALID_QUOTE:
		return "attribute value: invalid quote";
	case j1XML_RESULT_READ_PARSE_ERROR_ATTRIBUTE_VALUE_TOO_BIG:
		return "attribute value: too big";
	case j1XML_RESULT_READ_PARSE_ERROR_DECL_END_MISMATCH:
		return "decl end mismatch";
	case j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_DECL_END:
		return "no character after decl end";
	case j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_DECL_END:
		return "no angle after decl end";
	case j1XML_RESULT_READ_PARSE_ERROR_EMPTY_END_MISMATCH:
		return "empty end mismatch";
	case j1XML_RESULT_READ_PARSE_ERROR_NO_CHAR_AFTER_EMPTY_END:
		return "no character after empty end";
	case j1XML_RESULT_READ_PARSE_ERROR_NO_ANGLE_AFTER_EMPTY_END:
		return "no angle after empty end";
	case j1XML_RESULT_READ_PARSE_ERROR_DECL_MISSING_END:
		return "decl missing end";
	case j1XML_RESULT_READ_PARSE_ERROR_COMM_LOGIC:
		return "comment logic error";
	case j1XML_RESULT_READ_PARSE_ERROR_EMPTY_LOGIC:
		return "empty logic error";
	case j1XML_RESULT_READ_PARSE_ERROR_LOGIC:
		return "general parse logic error";

	case j1XML_RESULT_WRITE_NOT_OPENED_CORRECTLY:
		return "'file' not opened for write";
	case j1XML_RESULT_WRITE_NO_ROOM:
		return "'file' out of space on write";
	case j1XML_RESULT_WRITE_UNKNOWN_ETYPE:
		return "unknown etype for write";
	case j1XML_RESULT_WRITE_NO_POSSIBLE_QUOTE:
		return "no possible quote on attribute value";
	case j1XML_RESULT_WRITE_ETYPE_LOGIC:
		return "etype logic error";
	case j1XML_RESULT_WRITE_ERROR:
		return "write error";

	case j1XML_RESULT_PARSE_ELEM_INVALID:
		return "no matching element name";
	case j1XML_RESULT_PARSE_CMD_INVALID:
		return "no matching command";

	case j1XML_RESULT_PARSE_ATTR_INVALID:
		return "no matching attribute name";
	case j1XML_RESULT_PARSE_ATTR_DUPE:
		return "duplicate attribute";
	case j1XML_RESULT_PARSE_ATTR_MINLEN:
		return "attr value too short";
	case j1XML_RESULT_PARSE_ATTR_MAXLEN:
		return "attr value too long";
	case j1XML_RESULT_PARSE_ATTR_ATYPE:
		return "atyle logic error";
	case j1XML_RESULT_PARSE_ATTR_STRING_INVALID:
		return "attr val string invalid";
	case j1XML_RESULT_PARSE_ATTR_BYTE_INVALID:
		return "attr val byte invalid";
	case j1XML_RESULT_PARSE_ATTR_WORD_INVALID:
		return "attr val word invalid";
	case j1XML_RESULT_PARSE_ATTR_DWORD_INVALID:
		return "attr val dword invalid";
	case j1XML_RESULT_PARSE_ATTR_INT16_INVALID:
		return "attr val int16 invalid";
	case j1XML_RESULT_PARSE_ATTR_INT32_INVALID:
		return "attr val int32 invalid";
	case j1XML_RESULT_PARSE_ATTR_BOOL_INVALID:
		return "attr val bool invalid";
	case j1XML_RESULT_PARSE_ATTR_IP_INVALID:
		return "attr val ip invalid";
	case j1XML_RESULT_PARSE_ATTR_MAC_INVALID:
		return "attr val mac invalid";
	case j1XML_RESULT_PARSE_ESTACK_EXCEEDED:
		return "estack too deep";
	case j1XML_RESULT_PARSE_ESTACK_EXHAUSTED:
		return "estack stop with no start";
	case j1XML_RESULT_PARSE_ESTACK_MISMATCH:
		return "estack start/stop mismatch";
	case j1XML_RESULT_PARSE_ETYPE_ERROR:
		return "etype parse error";

	case j1XML_RESULT_VALIDATE_INVALID:
		return "validate invalid";
		
	case j1XML_RESULT_SET_ELEMENT_NAME_ERROR:
		return "set element name error";
	case j1XML_RESULT_SET_ATTRIBUTE_ERROR:
		return "set attribute error";
		
	case j1XML_RESULT__MAXVALUE__:
		break;
	}

	return "unknown result code";
}

//---------------------------------------------------------------------
//
//	j1XML_Element_etxtcpy() - copy the element text to a string buffer
//
//---------------------------------------------------------------------

void j1XML_Element_etxtcpy(char *bfr, int bfrSize, j1XML_ELEMENT *element)
{
	uint32	offset;
	char	c;

	offset = element->eStartOffset;

	while (--bfrSize)
	{
		if (j1XML_ReadChar(&offset, &c, false) == false) break;
	
		*bfr++ = c;
		offset++;

		if (element->eEndOffset) if (offset > element->eEndOffset) break;
	}

	*bfr = 0;
}

//---------------------------------------------------------------------
//
//	j1XMLSetElementName() - copy name from dictionary into an element
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_SetElementName(j1XML_ELEMENT *element, j1XML_DICT_ELEM const *dictElem)
{
	if (strlen(dictElem->name) > element->eNameSize - 1) return j1XML_RESULT_SET_ELEMENT_NAME_ERROR;
	strcpy(element->eNameBfr, dictElem->name);
	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XMLSetAttribute() - set name/value for an attribute in an element
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_SetAttribute(j1XML_ELEMENT *element, uint16 *aCnt, void *valPtr, j1XML_DICT_ATTR const *dictAttr)
{
	char	*aPtr;
	char	bfr[32];
	int32	value;

	if (strlen(dictAttr->name) > element->aNameSize - 1) return j1XML_RESULT_SET_ATTRIBUTE_ERROR;
	if (*aCnt >= element->aCount) return j1XML_RESULT_SET_ATTRIBUTE_ERROR;

	if (*aCnt == 0) memset(element->aBfr, 0, element->aCount * (element->aNameSize + element->aValueSize));
	
	aPtr = &element->aBfr[*aCnt * (element->aNameSize + element->aValueSize)];

	strcpy(aPtr, dictAttr->name);
	aPtr += element->aNameSize;

	switch (dictAttr->aType)
	{
	case j1XML_DICT_ATYPE_BARE:
		break;
		
	case j1XML_DICT_ATYPE_STRING:
		if (strlen(valPtr) > element->aValueSize - 1) return j1XML_RESULT_SET_ATTRIBUTE_ERROR;
		strcpy(aPtr, valPtr);
		break;

	case j1XML_DICT_ATYPE_CHAR:
		*aPtr++ = *(char *) valPtr;
		*aPtr   = 0;
		break;

	case j1XML_DICT_ATYPE_BYTE:
		j1_u8toa(*(uint8 *)valPtr, bfr);
		strcpy(aPtr, bfr);
		break;

	case j1XML_DICT_ATYPE_HEXBYTE:
		*aPtr++ = j1_btohexa_high(*(uint8 *)valPtr);
		*aPtr++ = j1_btohexa_low(*(uint8 *)valPtr);
		*aPtr   = 0;
		break;

	case j1XML_DICT_ATYPE_WORD:
		j1_u16toa(*(uint16 *)valPtr, bfr);
		strcpy(aPtr, bfr);
		break;

	case j1XML_DICT_ATYPE_DWORD:
		j1_u32toa(*(uint32 *)valPtr, bfr);
		strcpy(aPtr, bfr);
		break;

	case j1XML_DICT_ATYPE_INT16:
		value = *(int16 *)valPtr;
		if (value < 0) { value = -value; *aPtr++ = '-'; }
		j1_u32toa(value, bfr);
		strcpy(aPtr, bfr);
		break;

	case j1XML_DICT_ATYPE_INT32:
		value = *(int32 *)valPtr;
		if (value < 0) { value = -value; *aPtr++ = '-'; }
		j1_u32toa(value, bfr);
		strcpy(aPtr, bfr);
		break;

	case j1XML_DICT_ATYPE_BOOL:
		if (*(bool *)valPtr) *aPtr++ = '1';
						else *aPtr++ = '0';
		*aPtr = 0;
		break;

	case j1XML_DICT_ATYPE_IP:
		for (value = 0; value < 4; value++)
		{
			if (value) *aPtr++ = '.';
			j1_u16toa(((j1UINT32_VAL *)valPtr)->B[value], bfr);
			strcpy(aPtr, bfr);
			aPtr += strlen(aPtr);
		}
		break;

	case j1XML_DICT_ATYPE_MAC:
		for (value = 0; value < 6; value++)
		{
			if (value) *aPtr++ = ':';
			*aPtr++ = j1_btohexa_high(((char *)valPtr)[value]);
			*aPtr++ = j1_btohexa_low(((char *)valPtr)[value]);
		}
		break;
	}

	(*aCnt)++;
	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XMLStrToMAC() - convert an attribute value to mac array
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_StrToMAC(char *valPtr, int valLen, uint8 *mac)
{
//	uint16		octetVal;
	int			i;
	char		hi, lo, term;

	if (strlen(valPtr) != 17) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;

	for (i = 0; i < 6; i++)
	{
		hi   = TOUPPER(*valPtr++);
		lo   = TOUPPER(*valPtr++);
		term = *valPtr++;

		if (!ISXDIGIT(hi)) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
		if (!ISXDIGIT(lo)) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;

		if (i < 5)
		{
			if (term != ':') return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
		}
		else
		{
			if (term != 0x00) return j1XML_RESULT_PARSE_ATTR_MAC_INVALID;
		}

		hi -= '0'; if (hi > 9) hi -= 'A' - '9' - 1;
		lo -= '0'; if (lo > 9) lo -= 'A' - '9' - 1;
		mac[i] = hi << 4 | lo;
	}

	return j1XML_RESULT_OK;
}

//---------------------------------------------------------------------
//
//	j1XML_StrToIP() - convert an attribute value to an ip array
//
//---------------------------------------------------------------------

j1XML_RESULT j1XML_StrToIP(char *valPtr, int valLen, uint8 *ip)
{
		uint16		octetVal;
		char		*termPtr;

		// try 1st octet
		octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
		// if we point to not '.' we have error
		if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		ip[0] = octetVal;
		valPtr = ++termPtr;

		// try 2nd octet
		octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
		// if we point to not '.' we have error
		if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		ip[1] = octetVal;
		valPtr = ++termPtr;

		// try 3rd octet
		octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
		// if we point to not '.' we have error
		if (*termPtr != '.') return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		ip[2] = octetVal;
		valPtr = ++termPtr;

		// try 4th octet
		octetVal = (uint16) j1_strtoul(valPtr, &termPtr, 0);
		// if we don't point to '\0' we have error
		if (*termPtr) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		if (255 < octetVal) return j1XML_RESULT_PARSE_ATTR_IP_INVALID;
		ip[3] = octetVal;

		return j1XML_RESULT_OK;
}
